#include "plumpconfig.h"

int dbgapi_okay()
{
#ifdef MEDIUM_HAS_DBGAPI
	return PLUMP_SUCCESS;
#else
	return PLUMP_ERROR_NOTSUPPORTED;
#endif
}

#ifdef MEDIUM_HAS_DBGAPI

#include <windows.h>

typedef struct _dbgapi_extension {

	HANDLE process;

} dbgapi_extension_t;

plumpmedium_t dbgapi_getMedium()
{
	return PLUMP_MEDIUM_DBGAPI;
}

int dbgapi_open(plump_t *p, int pid, unsigned long access)
{
	dbgapi_extension_t *ext;
	DWORD actualAccess = 0;
	int ret = PLUMP_ERROR_FAILURE;

	do
	{
		// Allocate our medium extension
		if (!(p->mediumExtension = (void *)malloc(sizeof(dbgapi_extension_t))))
		{
			ret = PLUMP_ERROR_NOMEMORY;
			break;
		}

		ext = (dbgapi_extension_t *)p->mediumExtension;

		if (access & PLUMP_ACCESS_READ)
			actualAccess |= PROCESS_VM_READ;
		if (access & PLUMP_ACCESS_WRITE)
			actualAccess |= PROCESS_VM_WRITE;
		if (access & PLUMP_ACCESS_EXEC)
			actualAccess |= PROCESS_ALL_ACCESS;

		// Try to attach to the process
		if (!(ext->process = OpenProcess(actualAccess, FALSE, pid)))
		{
			switch (GetLastError())
			{
				case ERROR_ACCESS_DENIED:
					ret = PLUMP_ERROR_NOACCESS;
					break;
				default:
					ret = PLUMP_ERROR_NOSUCHPID;
					break;
			}

			break;
		}

		ret = PLUMP_SUCCESS;

	} while (0);

	return ret;
}

int dbgapi_close(plump_t *p)
{
	dbgapi_extension_t *ext = (dbgapi_extension_t *)p->mediumExtension;

	if (ext)
	{
		if (ext->process)
			CloseHandle(ext->process);

		free(ext);
	}

	p->mediumExtension = NULL;

	return PLUMP_SUCCESS;
}

int dbgapi_execContinue(plump_t *p, plumpaddr_t start, plumpaddr_t *stop, unsigned long flags)
{
	return PLUMP_ERROR_NOTSUPPORTED;
}

int dbgapi_execStop(plump_t *p)
{
	return PLUMP_ERROR_NOTSUPPORTED;
}

int dbgapi_memoryRead(plump_t *p, plumpaddr_t addr, void *buffer, unsigned long bufferLength, unsigned long *bytesRead)
{
	dbgapi_extension_t *ext = (dbgapi_extension_t *)p->mediumExtension;
	int ret;

	if (!ReadProcessMemory(ext->process, (LPCVOID)addr, buffer, bufferLength, bytesRead))
	{
		switch (GetLastError())
		{
			case ERROR_ACCESS_DENIED:
				ret = PLUMP_ERROR_NOACCESS;
				break;
			case ERROR_INVALID_ADDRESS:
				ret = PLUMP_ERROR_NOSUCHADDR;
				break;
			default:
				ret = PLUMP_ERROR_FAILURE;
				break;
		}
	}
	else
		ret = PLUMP_SUCCESS;

	return ret;
}

int dbgapi_memoryWrite(plump_t *p, plumpaddr_t addr, void *buffer, unsigned long bufferLength, unsigned long *bytesWritten)
{
	dbgapi_extension_t *ext = (dbgapi_extension_t *)p->mediumExtension;
	int ret;

	if (!WriteProcessMemory(ext->process, (LPVOID)addr, buffer, bufferLength, bytesWritten))
	{
		switch (GetLastError())
		{
			case ERROR_ACCESS_DENIED:
				ret = PLUMP_ERROR_NOACCESS;
				break;
			case ERROR_INVALID_ADDRESS:
				ret = PLUMP_ERROR_NOSUCHADDR;
				break;
			default:
				ret = PLUMP_ERROR_FAILURE;
				break;
		}
	}
	else
		ret = PLUMP_SUCCESS;

	return ret;
}

int dbgapi_memoryIsValid(plump_t *p, plumpaddr_t addr)
{
	char buf;

	return dbgapi_memoryRead(p, addr, &buf, 1, NULL);
}

int dbgapi_memoryGetEndOfSegment(plump_t *p, plumpaddr_t addr, plumpaddr_t *end)
{
	int ret = PLUMP_ERROR_NOTSUPPORTED;

	return ret;
}

int dbgapi_registersSet(plump_t *p, plumpreg_t *registers, unsigned long numRegisters)
{
	return PLUMP_ERROR_NOTSUPPORTED;
}

int dbgapi_registersGet(plump_t *p, plumpreg_t *registers, unsigned long numRegisters)
{
	return PLUMP_ERROR_NOTSUPPORTED;
}

#endif
